# simple example which shows processing data from LibreOffice spreadsheets

require 'dreader'

processor = Dreader::Engine.new

processor.options do
  first_row 2
  filename "cities_by_state.ods"
end

processor.column :state do |col|
  col.colref 'A'
end

processor.column :cities do |col|
  col.colref 'B'
  col.check do |data|
    data.class == Integer
  end
end

processor.mapping do |row|
  hash = Dreader::Util.simplify row
  puts "  states: #{hash[:state]} number of cities: #{hash[:cities]}"
end

printf "Loading the spreadsheet..."
processor.load
puts "done!"

puts "Checking errors..."
errors = processor.errors
if errors.size > 0
  puts "  Errors found:"
  processor.errors.each do |error|
    puts error
  end
else
  puts "  No errors found."
end
puts "done!"

puts "Processing the spreadsheet..."
processor.process
puts "... done"
    
  
