require 'dreader'

# this is the class which will contain all the data we read from the file
class City
  [:city, :state, :population, :lat, :lon].each do |var|
    attr_accessor var
  end

  def initialize(hash)
    hash.each do |k, v|
      self.send("#{k}=", v)
    end
  end
end

importer = Dreader::Engine.new

# read from us_cities.tsv, lines from 2 to 10 (included)
importer.options do
  filename "us_cities.tsv"
  first_row 2
  last_row  10
end

# these are the columns for which we only need to specify column and name
[[:city, 2], [:state, 3], [:latlon, 11]].each do |val|
  # val[0] -> :city
  # val[1] -> 2
  importer.column val[0] do |col|
    col.colref val[1]
    col.process do |val|
      val.strip
    end
  end
end

# the population column requires more work
importer.column :population do |col|
  col.colref 4

  # make "3,000" into 3000 (int)
  col.process do |value|
    value.gsub(",", "").to_i
  end

  col.check do |value|
    value > 0
  end

end

cities = []

importer.mapping do |row|
  # remove all additional information stored in each cell
  r = Dreader::Util.simplify row

  # make latlon into the lat, lon fields
  r[:lat], r[:lon] = r[:latlon].split(" ")

  # now r contains something like
  # {lat: ..., lon: ..., city: ..., state: ..., population: ..., latlon: ...}

  # remove fields which are not understood by the Cities class and
  # make a new instance
  cleaned = Dreader::Util.clean r, [:latlon]

  # you must declare an array cities before calling importer.process
  cities << City.new(cleaned)
end

# print to stdout what we told dreader to read
# (useful only for ... debugging!)
importer.debug n: 10

# check some other features of debug:
# disable processing and debug (e.g., to analyze the raw data read)
importer.debug process: false, check: false

# load and process
importer.load
cities = []
importer.process

# output everything to see whether it works
puts "First ten cities in the US (source Wikipedia)"
cities.each do |city|
  [:city, :state, :population, :lat, :lon].each do |var|
    puts "#{var.to_s.capitalize}: #{city.send(var)}"
  end
  puts ""
end
